/***********************************
 * Author: Matt Christian
 * 
 * Description: Implements a Game Class
 * 
 * Notes: For use with XNA Development Tutorials
 * 
 * If you use this for a project, please give credit!
 * http://matt.insidegamer.org/xnatutorials.aspx
 * 
 * ***********************************/
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        GameModel myHouse;
        GameCamera camera;
        GameSprite gameSprites;
        GameState gameStates;
        GameAudio gameAudio;

        float previousRotation = 0.0f;

        public struct keylog
        {
            public Keys _key;
            public bool _keyPressed;
        };

        keylog remKey;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            myHouse = new GameModel(this.Content, graphics, "Models\\houseModel");
            camera = new GameCamera(graphics);
            gameStates = new GameState();

            remKey._key = Keys.None;
            remKey._keyPressed = false;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            gameStates.ResetStates();
            gameStates.Loading = true;

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            gameSprites = new GameSprite(this.Content, spriteBatch);

            gameAudio = new GameAudio();


            gameSprites.AddSprite("RadarBG", "RadarBG", graphics.GraphicsDevice.Viewport.Width - 100, 100, 0.0f, 0.7f, true);

            gameSprites.AddSprite("RadarChar", "RadarChar", graphics.GraphicsDevice.Viewport.Width - 100, 100, 0.0f, 0.7f, true);

            gameSprites.AddSprite("4Buttons", "4Buttons", graphics.GraphicsDevice.Viewport.Width - 90, graphics.GraphicsDevice.Viewport.Height - 90, 0.0f, 1.0f, true);

            gameSprites.AddSprite("HealthBG", "HealthBG", 140, graphics.GraphicsDevice.Viewport.Height - 100, 0.0f, 1.0f, true);

            gameSprites.AddSprite("HealthBorder", "HealthBorder", 140, graphics.GraphicsDevice.Viewport.Height - 100, 0.0f, 1.0f, true);

            //Load Pause Screen
            gameSprites.AddSprite("PauseBG", "PauseBG", graphics.GraphicsDevice.Viewport.Width/2, graphics.GraphicsDevice.Viewport.Height/2, 0.0f, 1.0f, false);

            gameSprites.AddSprite("PauseMenuLargeBG", "PauseMenuLargeBG", graphics.GraphicsDevice.Viewport.Width/2, graphics.GraphicsDevice.Viewport.Height/2, 0.0f, 1.0f, false);

            gameSprites.AddSprite("PauseMenuExit", "PauseMenuExit", graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Height / 2 + 200, 0.0f, 1.0f, false);

            gameSprites.AddSprite("PauseMenuExitSelected", "PauseMenuExitSelected", graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Height / 2 + 200, 0.0f, 1.0f, false);            
            
            //Load Audio
            gameAudio.AddSound("rain");
            gameAudio.AddSound("pause");

            gameStates.ResetStates();
            gameStates.InGame = true;

            gameAudio.PlaySound("rain");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Displays Pause Menu
        /// </summary>
        public void ShowPauseMenu()
        {
            //Show Pause Screen
            gameSprites.UpdateSpriteVisibility("PauseBG", "PauseBG", true);

            gameSprites.UpdateSpriteVisibility("PauseMenuLargeBG", "PauseMenuLargeBG", true);

            gameSprites.UpdateSpriteVisibility("PauseMenuExit", "PauseMenuExit", true);
        }

        /// <summary>
        /// Hides Pause Menu
        /// </summary>
        public void HidePauseMenu()
        {
            //Hide Pause Screen
            gameSprites.UpdateSpriteVisibility("PauseBG", "PauseBG", false);

            gameSprites.UpdateSpriteVisibility("PauseMenuLargeBG", "PauseMenuLargeBG", false);

            gameSprites.UpdateSpriteVisibility("PauseMenuExit", "PauseMenuExit", false);

            gameSprites.UpdateSpriteVisibility("PauseMenuExitSelected", "PauseMenuExitSelected", false);
        }

        /// <summary>
        /// Checks any pause menu input
        /// </summary>
        public void CheckPauseMenu()
        {
            MouseState currMouse = Mouse.GetState();
            int mouseX = currMouse.X;
            int mouseY = currMouse.Y;

            Rectangle recExit = gameSprites.GetBoundingBox("PauseMenuExit");

            if ((mouseX >= recExit.Left) && (mouseX <= recExit.Right) &&
                (mouseY <= recExit.Bottom) && (mouseY >= recExit.Top))
            {
                if (gameSprites.IsSpriteVisible("PauseMenuExit"))
                {
                    gameSprites.UpdateSpriteVisibility("PauseMenuExit", "PauseMenuExit", false);
                    gameSprites.UpdateSpriteVisibility("PauseMenuExitSelected", "PauseMenuExitSelected", true);
                }

                if (currMouse.LeftButton == ButtonState.Pressed)
                {
                    //End game
                    this.Exit();
                }

            }
            else
            {
                if (gameSprites.IsSpriteVisible("PauseMenuExitSelected"))
                {
                    gameSprites.UpdateSpriteVisibility("PauseMenuExit", "PauseMenuExit", true);
                    gameSprites.UpdateSpriteVisibility("PauseMenuExitSelected", "PauseMenuExitSelected", false);
                }
            }
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState().IsKeyUp(Keys.Space) && (remKey._key == Keys.Space && remKey._keyPressed == true))
            {
                remKey._key = Keys.None;
                remKey._keyPressed = false;
            }

            //Exit game
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            if (gameStates.InGame)
            {
                //Move forward
                if (Keyboard.GetState().IsKeyDown(Keys.W))
                    camera.MoveForward();

                //Move Back
                if (Keyboard.GetState().IsKeyDown(Keys.S))
                    camera.MoveBack();

                //Strafe Left
                if (Keyboard.GetState().IsKeyDown(Keys.A))
                    camera.StrafeLeft();

                //Strafe Right
                if (Keyboard.GetState().IsKeyDown(Keys.D))
                    camera.StrafeRight();

                //Turn left
                if (Keyboard.GetState().IsKeyDown(Keys.Left))
                {
                    camera.turnAmt += camera.rotationSpeed;
                    camera.RotateCamera();
                }

                //Turn right
                if (Keyboard.GetState().IsKeyDown(Keys.Right))
                {
                    camera.turnAmt -= camera.rotationSpeed;
                    camera.RotateCamera();
                }

                //Pause Game
                if (Keyboard.GetState().IsKeyDown(Keys.Space) && (remKey._key != Keys.Space && remKey._keyPressed == false))
                {
                    remKey._key = Keys.Space;
                    remKey._keyPressed = true;

                    gameAudio.PauseSound("rain");
                    gameAudio.PlaySound("pause");

                    ShowPauseMenu();
                    this.IsMouseVisible = true;

                    gameStates.ResetStates();
                    gameStates.Paused = true;
                }
            }

            if (gameStates.Paused)
            {
                CheckPauseMenu();

                if (Keyboard.GetState().IsKeyDown(Keys.Space) && (remKey._key != Keys.Space && remKey._keyPressed == false))
                {
                    remKey._key = Keys.Space;
                    remKey._keyPressed = true;

                    gameAudio.PlaySound("pause");
                    gameAudio.UnpauseSound("rain");

                    HidePauseMenu();
                    this.IsMouseVisible = false;

                    gameStates.ResetStates();
                    gameStates.InGame = true;
                }
            }

            gameAudio.UpdateAudio();

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            myHouse.SetPosition(0.0f, 0.0f, 5.0f);
            //Render Object
            myHouse.Render(camera.GetViewMatrix(), camera.GetProjectionMatrix());

            if (previousRotation != camera.GetRotation())
            {
                gameSprites.UpdateSpriteRotation("RadarBG", "RadarBG", camera.GetRotation());
                previousRotation = camera.GetRotation();
            }

            gameSprites.DrawSprites();

            base.Draw(gameTime);
        }
    }
}
